from contextlib import asynccontextmanager
import logging
from typing import Any

#import colorama
#colorama.init()

from fastapi import FastAPI, responses

from .api import api
from .geoapi import api as geoapi
from .config import conf
from .registry import registry, ModelRegistry
from .redis_tools import setup_redis, shutdown_redis, setup_redis_cache
from .live import setup_live

logging.basicConfig(level=conf.gisaf.debugLevel)
logger = logging.getLogger(__name__)

## Subclass FastAPI to add attributes to be used globally, ie. registry
class GisafExtra:
    registry: ModelRegistry
    #raw_survey_models: dict[str, Any] = {}
    #survey_models: dict[str, Any] = {}


class GisafFastAPI(FastAPI):
    gisaf_extra: GisafExtra

@asynccontextmanager
async def lifespan(app: FastAPI):
    await registry.make_registry()
    await setup_redis()
    await setup_live()
    yield
    await shutdown_redis()

app = FastAPI(
    debug=False,
    title=conf.gisaf.title,
    version=conf.version,
    lifespan=lifespan,
    default_response_class=responses.ORJSONResponse,
)

app.mount('/v2', api)
app.mount('/gj', geoapi)