#!/usr/bin/env python
"""
Gisaf job scheduler, orchestrating the background tasks
like remote device data collection, etc.
"""
import logging
from contextlib import asynccontextmanager

from starlette.routing import Mount
from fastapi.middleware.cors import CORSMiddleware
from apscheduler.schedulers.asyncio import AsyncIOScheduler

from .config import conf
from .ipynb_tools import gisaf
from .scheduler import GSFastAPI, js, startup, Settings
from .scheduler_web import app as sched_app


formatter = logging.Formatter(
    "%(asctime)s:%(levelname)s:%(name)s:%(message)s",
    "%Y-%m-%d %H:%M:%S"
)
for handler in logging.root.handlers:
    handler.setFormatter(formatter)

logging.basicConfig(level=conf.gisaf.debugLevel)
logger = logging.getLogger('gisaf.scheduler_application')


@asynccontextmanager
async def lifespan(app: GSFastAPI):
    '''
    Handle startup and shutdown: setup scheduler, etc
    '''
    ## Startup
    await gisaf.setup()
    await startup(settings)
    js.start()
    yield
    ## Shutdown
    pass


settings = Settings()
app = GSFastAPI(
    title=settings.app_name,
    lifespan=lifespan,
)
app.add_middleware(
    CORSMiddleware,
    allow_origins=['*'],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)
app.mount('/_sched', sched_app)
app.mount('/sched', sched_app)
